@@ -1,8 +1,3 @@
-# Copyright (c) 2011 CentralNic Ltd. All rights reserved. This program is
-# free software; you can redistribute it and/or modify it under the same
-# terms as Perl itself.
-# 
-# $Id: Contact.pm,v 1.3 2011/12/14 12:02:08 gavin Exp $
 package Net::EPP::Frame::Command::Create::Contact;
 use base qw(Net::EPP::Frame::Command::Create);
 use Net::EPP::Frame::ObjectSpec;
@@ -156,25 +151,4 @@ sub addEl {
 	
 }
 
-=pod
-
-=head1 AUTHOR
-
-CentralNic Ltd (http://www.centralnic.com/).
-
-=head1 COPYRIGHT
-
-This module is (c) 2011 CentralNic Ltd. This module is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Net::EPP::Frame>
-
-=back
-
-=cut
-
 1;
