@@ -1,8 +1,3 @@
-# Copyright (c) 2009 CentralNic Ltd. All rights reserved. This program is
-# free software; you can redistribute it and/or modify it under the same
-# terms as Perl itself.
-# 
-# $Id: Login.pm,v 1.5 2007/12/03 11:44:52 gavin Exp $
 package Net::EPP::Frame::Command::Login;
 use base qw(Net::EPP::Frame::Command);
 use strict;
@@ -29,6 +24,10 @@ sub _addCommandElements {
 	$self->getNode('login')->addChild($self->createElement('clID'));
 	$self->getNode('login')->addChild($self->createElement('pw'));
 	$self->getNode('login')->addChild($self->createElement('options'));
+
+	$self->getNode('options')->addChild($self->createElement('version'));
+	$self->getNode('options')->addChild($self->createElement('lang'));
+
 	$self->getNode('login')->addChild($self->createElement('svcs'));
 }
 
@@ -46,6 +45,11 @@ C<E<lt>clIDE<gt>> element.
 This method returns the L<XML::LibXML::Element> object corresponding to the
 C<E<lt>pwE<gt>> element.
 
+	my $node = $frame->newPW;
+
+This method returns the L<XML::LibXML::Element> object corresponding to the
+C<E<lt>newPWE<gt>> element.
+
 	my $node = $frame->svcs;
 
 This method returns the L<XML::LibXML::Element> object corresponding to the
@@ -56,32 +60,24 @@ C<E<lt>svcsE<gt>> element.
 This method returns the L<XML::LibXML::Element> object corresponding to the
 C<E<lt>optionsE<gt>> element.
 
+	my $node = $frame->version;
+
+This method returns the L<XML::LibXML::Element> object corresponding to the
+C<E<lt>versionE<gt>> element.
+
+	my $node = $frame->lang;
+
+This method returns the L<XML::LibXML::Element> object corresponding to the
+C<E<lt>langE<gt>> element.
+
 =cut
 
 sub clID { $_[0]->getNode('clID') }
 sub pw { $_[0]->getNode('pw') }
+sub newPW { $_[0]->getNode('newPW') }
 sub svcs { $_[0]->getNode('svcs') }
 sub options { $_[0]->getNode('options') }
-
-=pod
-
-=head1 AUTHOR
-
-CentralNic Ltd (http://www.centralnic.com/).
-
-=head1 COPYRIGHT
-
-This module is (c) 2007 CentralNic Ltd. This module is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Net::EPP::Frame>
-
-=back
-
-=cut
+sub version { $_[0]->getNode('version') }
+sub lang { $_[0]->getNode('lang') }
 
 1;
