@@ -1,8 +1,3 @@
-# Copyright (c) 2011 CentralNic Ltd. All rights reserved. This program is
-# free software; you can redistribute it and/or modify it under the same
-# terms as Perl itself.
-# 
-# $Id: Domain.pm,v 1.1 2011/12/03 11:44:52 gavin Exp $
 package Net::EPP::Frame::Command::Renew::Domain;
 use base qw(Net::EPP::Frame::Command::Renew);
 use Net::EPP::Frame::ObjectSpec;
@@ -126,26 +121,4 @@ sub setCurExpDate {
 	return 1;
 }
 
-
-=pod
-
-=head1 AUTHOR
-
-CentralNic Ltd (http://www.centralnic.com/).
-
-=head1 COPYRIGHT
-
-This module is (c) 2011 CentralNic Ltd. This module is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Net::EPP::Frame>
-
-=back
-
-=cut
-
 1;
