@@ -1,11 +1,7 @@
-# Copyright (c) 2007 CentralNic Ltd. All rights reserved. This program is
-# free software; you can redistribute it and/or modify it under the same
-# terms as Perl itself.
-# 
-# $Id: Update.pm,v 1.5 2007/12/14 12:02:08 gavin Exp $
 package Net::EPP::Frame::Command::Update;
 use Net::EPP::Frame::Command::Update::Contact;
 use Net::EPP::Frame::Command::Update::Domain;
+use Net::EPP::Frame::Command::Update::Host;
 use base qw(Net::EPP::Frame::Command);
 use strict;
 
@@ -62,23 +58,6 @@ sub chg {
 These methods return the elements that should be used to contain the changes
 to be made to the object (ie C<domain:add>, C<domain:rem>, C<domain:chg>).
 
-=head1 AUTHOR
-
-CentralNic Ltd (http://www.centralnic.com/).
-
-=head1 COPYRIGHT
-
-This module is (c) 2007 CentralNic Ltd. This module is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Net::EPP::Frame>
-
-=back
-
 =cut
 
 1;
