@@ -1,8 +1,3 @@
-# Copyright (c) 2012 CentralNic Ltd. All rights reserved. This program is
-# free software; you can redistribute it and/or modify it under the same
-# terms as Perl itself.
-# 
-# $Id: Contact.pm,v 1.3 2011/12/03 11:44:52 gavin Exp $
 package Net::EPP::Frame::Command::Update::Contact;
 use base qw(Net::EPP::Frame::Command::Update);
 use Net::EPP::Frame::ObjectSpec;
@@ -92,6 +87,45 @@ sub setContact {
 	return 1;
 }
 
+=pod
+
+	$frame->chgVoice($voice);
+
+Change the contacts voice number.
+
+=cut
+
+sub chgVoice {
+	my ($self, $voice) = @_;
+	return $self->addEl('voice', $voice);
+}
+
+=pod
+
+	$frame->chgFax($fax);
+
+Change the contacts voice number.
+
+=cut
+
+sub chgFax {
+	my ($self, $fax) = @_;
+	return $self->addEl('fax', $fax);
+}
+
+=pod
+
+	$frame->chgEmail($email);
+
+Change the contacts email.
+
+=cut
+
+sub chgEmail {
+	my ($self, $email) = @_;
+	return $self->addEl('email', $email);
+}
+
 =pod
 
 	$frame->addStatus($type, $info);
@@ -186,26 +220,16 @@ sub chgAuthInfo {
 	return 1;
 }
 
+sub addEl {
+	my ($self, $name, $value) = @_;
 
-=pod
+	my $el = $self->createElement('contact:'.$name);
+	$el->appendText($value) if defined($value);
 
-=head1 AUTHOR
+	$self->getElementsByLocalName('contact:chg')->shift->appendChild($el);
 
-CentralNic Ltd (http://www.centralnic.com/).
+	return $el;
 
-=head1 COPYRIGHT
-
-This module is (c) 2012 CentralNic Ltd. This module is free software; you can
-redistribute it and/or modify it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Net::EPP::Frame>
-
-=back
-
-=cut
+}
 
 1;
